#!/bin/bash

module="lacp.o"
nicmodule="dpm.o"
device="lacp"
group="sys"
mode="664"

CONFPATH=/link_aggr/utility/lacpconfig

MaxTxDesc=200;
MaxRxDesc=200;
SetAutoNeg=1;
RxBufSize=2048;
MediaSpeed=1000;
NCBit=1;
vlan=0;

usage() {
	cat <<__EOF__
$0  -l|-u|-c [-n NCBit] [-t MaxTxDesc] [-r MaxRxDesc] [-a SetAutoNeg] [-b RxBufSize] [-s MediaSpeed] [-v vlan] [-p config_file_path]

	-l : Load LACP and NIC driver
	-u : Unload LACP and NIC driver
	-c : Configure Aggregation
__EOF__

}

if [ $# -eq 0 ]
then
	usage
	exit 1
fi

while getopts lucn:p:t:r:b:s:a:v: opt
do
	case $opt in
	n) NCBit=$OPTARG;;
	p) CONFPATH="$OPTARG";;
	t) MaxTxDesc=$OPTARG;;
	r) MaxRxDesc=$OPTARG;;
	b) RxBufSize=$OPTARG;;
	s) MediaSpeed=$OPTARG;;
	a) SetAutoNeg=$OPTARG;;
	v) vlan=$OPTARG;;
	l) Param="load";;
	u) Param="unload";;
	c) Param="config";;
	\?) usage
		exit 1;;
	esac
done
case $Param in
	load) 
		/sbin/insmod -f /link_aggr/lacp/$module || exit 1

		/bin/rm -rf /dev/${device}0

		major=`/bin/cat /proc/devices | /bin/awk "\\$2==\"$device\" {print \\$1}"`

		/bin/mknod /dev/${device}0 c $major 0

		/bin/chgrp $group /dev/${device}?
		/bin/chmod $mode  /dev/${device}?

		/sbin/insmod -f /link_aggr/nic/$nicmodule MaxTxDesc=$MaxTxDesc MaxRxDesc=$MaxRxDesc RxBufSize=$RxBufSize MediaSpeed=$MediaSpeed NCBit=$NCBit SetAutoNeg=$SetAutoNeg vlan=$vlan|| exit 1

		;;

	config)
		/link_aggr/utility/lacpstart $CONFPATH
		;;
	unload)
		for interface in `/sbin/ifconfig | grep eth | awk '{print $1}'`
		do
			/sbin/ifconfig ${interface} down
		done
		/sbin/rmmod dpm
		/sbin/rmmod lacp
		;;
	*)
		usage
		exit 1
esac
